@@ -2,13 +2,21 @@
 This is the main module for PyBlosxom functionality.  PyBlosxom's setup 
 and default handlers are defined here.
 """
-from __future__ import nested_scopes
-import os, time, re, sys, StringIO
+
+# Python imports
+from __future__ import nested_scopes, generators
+import os, time, re, sys
+import cgi
+try: from cStringIO import StringIO
+except ImportError: from StringIO import StringIO
+
+# Pyblosxom imports
 import tools
 from entries.fileentry import FileEntry
 
-VERSION = "1.0.0"
-VERSION_DATE = VERSION + " (May 24, 2004)"
+
+VERSION = "1.2"
+VERSION_DATE = VERSION + " 3/25/2005"
 VERSION_SPLIT = tuple(VERSION.split('.'))
 
 class PyBlosxom:
@@ -18,14 +26,23 @@ class PyBlosxom:
     request through all the steps until the output is rendered and
     we're complete.
     """
-    def __init__(self, request):
+    def __init__(self, config, environ, data=None):
         """
-        Sets the request.
+        Sets configuration and environment.
+        Creates the L{Request} object.
 
-        @param request: A L{Pyblosxom.pyblosxom.Request} object
-        @type request: L{Pyblosxom.pyblosxom.Request} object
+        @param config: A dict containing the configuration variables.
+        @type config: dict
+
+        @param environ: A dict containing the environment variables.
+        @type environ: dict
+
+        @param data: A dict containing data variables.
+        @type data: dict
         """
-        self._request = request
+        config['pyblosxom_name'] = "pyblosxom"
+        config['pyblosxom_version'] = VERSION_DATE
+        self._request = Request(config, environ, data)
 
     def initialize(self):
         """
@@ -45,12 +62,15 @@ class PyBlosxom:
         # Get our URL and configure the base_url param
         if pyhttp.has_key('SCRIPT_NAME'):
             if not config.has_key('base_url'):
-                config['base_url'] = 'http://%s%s' % (pyhttp['HTTP_HOST'], pyhttp['SCRIPT_NAME'])
+				# allow http and https
+                config['base_url'] = '%s://%s%s' % (pyhttp['wsgi.url_scheme'], pyhttp['HTTP_HOST'], pyhttp['SCRIPT_NAME'])
         else:
             config['base_url'] = config.get('base_url', '')
 
-        if config["datadir"].endswith("\\") or config["datadir"].endswith("/"):
-            config['datadir'] = config['datadir'][:-1]
+        datadir = config["datadir"]
+        if datadir.endswith("/") or datadir.endswith("\\"):
+            datadir = datadir[:-1]
+            config['datadir'] = datadir
 
         # import and initialize plugins
         import plugin_utils
@@ -63,15 +83,39 @@ class PyBlosxom:
                                         mappingfunc=lambda x,y:y,
                                         defaultfunc=lambda x:x)
 
-    def run(self):
+    def getRequest(self):
         """
-        Main loop for pyblosxom.  This should be called _after_
-        initialization.  This method will run the handle callback
+        Returns the L{Request} object.
+        
+        @returns: the request object 
+        @rtype: L{Request}
+        """
+        return self._request
+
+    def getResponse(self):
+        """
+        Returns the L{Response} object which handles all output 
+        related functionality.
+        
+        @see: L{Response}
+        @returns: the reponse object 
+        @rtype: L{Response}
+        """
+        return self._request.getResponse()
+
+    def run(self, static=False):
+        """
+        Main loop for pyblosxom.  This method will run the handle callback
         to allow registered handlers to handle the request.  If nothing
         handles the request, then we use the default_blosxom_handler.
         """
         self.initialize()
 
+        # buffer the input stream in a StringIO instance if dynamic rendering is used.
+        # This is done to have a known/consistent way of accessing incomming data.
+        if static == False:
+            self.getRequest().buffer_input_stream()
+
         # run the start callback
         tools.run_callback("start", {'request': self._request})
 
@@ -189,7 +233,6 @@ class PyBlosxom:
                 mtime = time.localtime(mtime)
                 year = time.strftime("%Y", mtime)
                 month = time.strftime("%m", mtime)
-                # monthname = tools.num2month[month]
                 day = time.strftime("%d", mtime)
 
                 dates[year] = 1
@@ -199,6 +242,7 @@ class PyBlosxom:
                     dates[year + "/" + month + "/" + day] = 1
 
                 if monthnames:
+                    monthname = tools.num2month[month]
                     dates[year + "/" + monthname] = 1
                     dates[year + "/" + monthname + "/" + day] = 1
 
@@ -256,11 +300,35 @@ class PyBlosxom:
         print "building %s files." % len(renderme)
 
         for url, q in renderme:
+            url = url.replace(os.sep, "/")
             print "rendering '%s' ..." % url
             tools.render_url(config, url, q)
 
 
-class Request:
+    def testInstallation(self):
+        test_installation(self._request)
+
+
+class EnvDict(dict):
+    """
+    Wrapper arround a dict to provide a backwards compatible way
+    to get the L{form<cgi.FieldStorage>} with syntax as:
+    request.getHttp()['form'] 
+    instead of:
+    request.getForm()
+    """
+    def __init__(self, request, env):
+        self._request = request
+        for key in env:
+            self[key] = env[key]
+
+    def __getitem__(self, key):
+        if key == "form":
+            return self._request.getForm()
+        else:
+            return dict.__getitem__(self, key)   
+
+class Request(object):
     """
     This class holds the PyBlosxom request.  It holds configuration
     information, HTTP/CGI information, and data that we calculate
@@ -271,18 +339,126 @@ class Request:
     PyBlosxom instance which will do further manipulation on the
     Request instance.
     """
-    def __init__(self):
+    def __init__(self, config, environ, data):
+        """
+        Sets configuration and environment.
+        Creates the L{Response} object which handles all output 
+        related functionality.
+        
+        @param config: A dict containing the configuration variables.
+        @type config: dict
+
+        @param environ: A dict containing the environment variables.
+        @type environ: dict
+
+        @param data: A dict containing data variables.
+        @type data: dict
+        """
         # this holds configuration data that the user changes 
         # in config.py
-        self._configuration = {}
+        self._configuration = config
 
         # this holds HTTP/CGI oriented data specific to the request
         # and the environment in which the request was created
-        self._http = {}
+        #self._http = environ
+        self._http = EnvDict(self, environ)
 
         # this holds run-time data which gets created and transformed
         # by pyblosxom during execution
-        self._data = {}
+        if data == None:
+            self._data = dict()
+        else:
+            self._data = data
+
+        # this holds the input stream.
+        # initialized for dynamic rendering in Pyblosxom.run.
+        # for static rendering there is no input stream.
+        self._in = StringIO()
+        # copy methods to the Request object.
+        self.__copy_members()
+
+        # this holds the FieldStorage instance.
+        # initialized when request.getForm is called the first time
+        self._form = None
+        
+        # create and set the Response
+        self.setResponse(Response(self))
+
+
+    def __copy_members(self):
+        """
+        Copies methods from the underlying input stream to the request object.
+        """
+        props = ['read', 'readline', 'readlines', 'seek', 'tell']
+        for prop in props:
+            setattr(self, prop, getattr(self._in, prop))
+
+    def __iter__(self):
+        """
+        Can't copy the __iter__ method over from the StringIO instance cause
+        iter looks for the method in the class instead of the instance.
+        So can't do this with __copy_members, have to define it seperatly.
+        See http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/252151
+        """
+        return self._in
+
+    def buffer_input_stream(self):
+        """
+        Buffer the input stream in a StringIO instance.
+        This is done to have a known/consistent way of accessing incomming data.
+        For example the input stream passed by mod_python does not offer the same 
+        functionallity as sys.stdin.
+        """
+        # TODO: tests on memory consumption when uploading huge files
+        input = self.getHttp()['wsgi.input']
+        self._in.write(input.read())
+        # rewind to start
+        self._in.seek(0)
+
+    def setResponse(self, response):
+        """
+        Sets the L{Response} object.
+
+        @param response: A pyblosxom Response object
+        @type response: L{Response}
+        """
+        self._response = response
+        # for backwards compatibility
+        self.getConfiguration()['stdoutput'] = response
+
+    def getResponse(self):
+        """
+        Returns the L{Response} object which handles all output 
+        related functionality.
+        
+        @returns: L{Response}
+        """
+        return self._response
+
+    def __getForm(self):
+        """
+        Parses and returns the form data submitted by the client.
+        Rewinds the input buffer after calling cgi.FieldStorage.
+
+        @returns: L{cgi.FieldStorage}
+        """        
+        form = cgi.FieldStorage(fp=self._in, environ=self._http, keep_blank_values=0)
+        # rewind the input buffer
+        self._in.seek(0)
+        return form
+
+    def getForm(self):
+        """
+        Returns the form data submitted by the client.
+        The L{form<cgi.FieldStorage>} instance is created
+        only when requested to prevent overhead and unnecessary
+        consumption of the input stream.
+
+        @returns: L{cgi.FieldStorage}
+        """
+        if self._form == None:
+            self._form = self.__getForm()
+        return self._form
 
     def getConfiguration(self):
         """
@@ -384,11 +560,129 @@ class Request:
         return "Request"
 
 
+class Response(object):
+    """
+    Response class to handle all output related tasks in one place.
+
+    This class is basically a wrapper arround a StringIO instance.
+    It also provides methods for managing http headers.
+    """
+
+    def __init__(self, request):
+        """
+        Sets the L{Request} object that leaded to this response.
+        Creates a L{StringIO} that is used as a output buffer.
+        
+        @param request: request object.
+        @type request: L{Request}
+        """
+        self._request = request
+        self._out = StringIO()
+        self._headers_sent = False
+        self.headers = {}
+        self.status = "200 OK"
+        self.__copy_members()
+    
+    def __copy_members(self):
+        """
+        Copies methods from the underlying output buffer to the response object.
+        """
+        props = ['close', 'flush', 
+            'read', 'readline', 'readlines', 'seek', 'tell',
+            'write', 'writelines']
+        for prop in props:
+            setattr(self, prop, getattr(self._out, prop))
+
+    def __iter__(self):
+        """
+        Can't copy the __iter__ method over from the StringIO instance cause
+        iter looks for the method in the class instead of the instance.
+        So can't do this with __copy_members, have to define it seperatly.
+        See http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/252151
+        """
+        return self._out
+
+    def setStatus(self, status):
+        """
+        Sets the status code for this response.
+        
+        @param status: A status code and message like '200 OK'.
+        @type status: str
+        """
+        self.status = status
+
+    def getStatus(self):
+        """
+        Returns the status code and message of this response.
+        
+        @returns: str
+        """
+        return self.status
+
+    def addHeader(self, *args):
+        """
+        Populates the HTTP header with lines of text.
+        Sets the status code on this response object if the given argument
+        list containes a 'Status' header.
+
+        @param args: Paired list of headers
+        @type args: argument lists
+        @raises ValueError: This happens when the parameters are not correct
+        """
+        args = list(args)
+        if not len(args) % 2:
+            while args:
+                key = args.pop(0).strip()
+                if key.find(' ') != -1 or key.find(':') != -1:
+                    raise ValueError, 'There should be no spaces in header keys'
+                value = args.pop(0).strip()
+
+                if key.lower() == "status":
+                    self.setStatus(str(value))
+                else:
+                    self.headers.update({key: str(value)})
+        else:
+            raise ValueError, 'Headers recieved are not in the correct form'
+
+    def getHeaders(self):
+        """
+        Returns the headers of this response.
+        
+        @returns: the HTTP response headers
+        @rtype: dict
+        """
+        return self.headers
+
+    def sendHeaders(self, out):
+        """
+        Send HTTP Headers to the given output stream.
+
+        @param out: File like object
+        @type out: file
+        """
+        out.write("Status: %s\n" % self.status)
+        out.write('\n'.join(['%s: %s' % (x, self.headers[x]) 
+                for x in self.headers.keys()]))
+        out.write('\n\n')
+        self._headers_sent = True
+
+    def sendBody(self, out):
+        """
+        Send the response body to the given output stream.
+
+        @param out: File like object
+        @type out: file
+        """
+        #if not self._headers_sent:
+        #    self.sendHeaders(out)
+        self.seek(0)
+        out.write(self.read())
+
+
 def blosxom_handler(request):
     """
     This is the default blosxom handler.
     """
-    import cgi
 
     config = request.getConfiguration()
     data = request.getData()
@@ -413,7 +707,6 @@ def blosxom_handler(request):
 
     data['renderer'] = r
 
-    request.addHttp( {"form": cgi.FieldStorage() } )
     # process the path info to determine what kind of blog entry(ies) 
     # this is
     tools.run_callback("pathinfo",
@@ -545,23 +838,29 @@ def blosxom_file_list_handler(args):
 
     entrylist = []
     for ourfile in filelist:
-        entry = FileEntry(request, ourfile, data['root_datadir'])
-        entrylist.append((entry._mtime, entry))
+        e = FileEntry(request, ourfile, data['root_datadir'])
+        entrylist.append((e._mtime, e))
 
     # this sorts entries by mtime in reverse order.  entries that have
     # no mtime get sorted to the top.
     entrylist.sort()
     entrylist.reverse()
-    entrylist = [x[1] for x in entrylist]
     
     # Match dates with files if applicable
     if data['pi_yr']:
+        # This is called when a date has been requested, e.g. /some/category/2004/Sep
         month = (data['pi_mo'] in tools.month2num.keys() and tools.month2num[data['pi_mo']] or data['pi_mo'])
         matchstr = "^" + data["pi_yr"] + month + data["pi_da"]
-        valid_list = [x for x in entrylist if re.match(matchstr, x['fulltime'])]
+        valid_list = [x for x in entrylist if re.match(matchstr, x[1]._fulltime)]
     else:
         valid_list = entrylist
 
+    # This is the maximum number of entries we can show (zero indicates show all entries)
+    max = config.get("num_entries", 0)
+    if max:
+        valid_list = valid_list[:max]
+    valid_list = [x[1] for x in valid_list]
+
     return valid_list
 
 def blosxom_process_path_info(args):
@@ -585,9 +884,9 @@ def blosxom_process_path_info(args):
     data = request.getData()
     pyhttp = request.getHttp()
 
-    form = pyhttp["form"]
+    form = request.getForm()
     data['flavour'] = (form.has_key('flav') and form['flav'].value or 
-            config.get('defaultFlavour', 'html'))
+            config.get('default_flavour', 'html'))
 
     path_info = []
     data['pi_yr'] = ''
@@ -605,7 +904,7 @@ def blosxom_process_path_info(args):
     for path_data in path_info:
         if not path_data:
             continue
-        elif len(path_data) == 4 and path_data.isdigit():
+        elif tools.is_year(path_data):
             # We got a hot date here guys :)
             got_date = 1
             break
@@ -636,10 +935,13 @@ def blosxom_process_path_info(args):
     
     data['bl_type'] = ''
 
+    if data['pi_bl'] != '':
+        data['blog_title_with_path'] = '%s : %s' % (config['blog_title'], data['pi_bl'])
+    else:
+        data['blog_title_with_path'] = config["blog_title"]
+
     # If all we've got is a directory, things are simple
     if os.path.isdir(blog_result):
-        if data['pi_bl'] != '':
-            config['blog_title'] += ' : %s' % data['pi_bl']
         data['root_datadir'] = blog_result
         data['bl_type'] = 'dir'
 
@@ -649,7 +951,6 @@ def blosxom_process_path_info(args):
 
         ext = tools.what_ext(data["extensions"].keys(), blog_result)
         if ext:
-            config['blog_title'] += ' : %s' % data['pi_bl']
             data['bl_type'] = 'file'
             data['root_datadir'] = blog_result + '.' + ext
 
@@ -662,14 +963,13 @@ def blosxom_process_path_info(args):
             if fileext:
                 data['flavour'] = ext[1:]
                 data['root_datadir'] = filename + '.' + fileext
-                config['blog_title'] += ' : %s' % data['pi_bl']
                 data['bl_type'] = 'file'
 
             elif (os.path.basename(filename) == 'index' and os.path.isdir(dirname)):
                 # blanket flavours?
                 data['flavour'] = ext[1:]
-                if data['pi_bl'] != '':
-                    config['blog_title'] += ' : %s' % os.path.dirname(data['pi_bl'])
+                if os.path.dirname(data['pi_bl']) != '':
+                    config['blog_title'] = '%s : %s' % (config['blog_title'], os.path.dirname(data['pi_bl']))
                 data['root_datadir'] = dirname
                 data['bl_type'] = 'dir'
                 
@@ -753,7 +1053,6 @@ def test_installation(request):
     print "]] checking datadir [["
 
     # DATADIR
-    # FIXME - we should check permissions here?
     if not os.path.isdir(config["datadir"]):
         print "datadir '%s' does not exist." % config["datadir"]          
         print "You need to create your datadir and give it appropriate"
@@ -762,7 +1061,8 @@ def test_installation(request):
         print "This must be done before we can go further.  Exiting."
         return
 
-    print "PASS: datadir is fine."
+    print "PASS: datadir is there.  Note: this does not check whether"
+    print "      your webserver has permissions to view files therein!"
 
     print "------"
     print "Now we're going to verify your plugin configuration."
@@ -778,6 +1078,7 @@ def test_installation(request):
         for mem in plugin_utils.plugins:
             if "verify_installation" in dir(mem):
                 print "=== plugin: '%s'" % mem.__name__
+                print "    file: %s" % mem.__file__
 
                 if "__version__" in dir(mem):
                     print "    version: %s" % mem.__version__
@@ -793,7 +1094,7 @@ def test_installation(request):
                     print " FAIL!!! ", error_message
 
             else:
-                no_verification_support.append(mem.__name__)
+                no_verification_support.append( "'%s' (%s)" % (mem.__name__, mem.__file__))
 
         if len(no_verification_support) > 0:
             print ""
