@@ -205,7 +205,7 @@ def Walk( request, root='.', recurse=0, pattern='', return_folders=0 ):
         ext = request.getData()['extensions']
         pattern = re.compile(r'.*\.(' + '|'.join(ext.keys()) + r')$')
 
-    ignore = request.getConfiguration().get("ignore", None)
+    ignore = request.getConfiguration().get("ignore_directories", None)
     if type(ignore) == type(""): ignore = [ignore]
     if ignore:
         for i in range(0, len(ignore)):
@@ -220,9 +220,9 @@ def Walk( request, root='.', recurse=0, pattern='', return_folders=0 ):
     except os.error:
         return []
 
-    return _walk_internal(root, recurse, pattern, ignorere, return_folders)
+    return __walk_internal(root, recurse, pattern, ignorere, return_folders)
 
-def _walk_internal( root, recurse, pattern, ignorere, return_folders ):
+def __walk_internal( root, recurse, pattern, ignorere, return_folders ):
     # initialize
     result = []
 
@@ -248,7 +248,7 @@ def _walk_internal( root, recurse, pattern, ignorere, return_folders ):
                     not os.path.islink(fullname) and \
                     (not ignorere or not ignorere.match(fullname)):
                 result = result + \
-                         _walk_internal(fullname, 
+                         __walk_internal(fullname, 
                                         (recurse > 1 and recurse -  1 or 0), 
                                         pattern, ignorere, return_folders)
 
@@ -256,35 +256,36 @@ def _walk_internal( root, recurse, pattern, ignorere, return_folders ):
 
 
 def filestat(request, filename):     
-     """     
-     Returns the filestat on a given file.  We store the filestat    
-     in case we've already retrieved it this time.   
+    """     
+    Returns the filestat on a given file.  We store the filestat    
+    in case we've already retrieved it this time.   
+    
+    @param request: the Pyblosxom Request object    
+    @type  request: Request     
+    
+    @param filename: the name of the file to stat   
+    @type  filename: string     
+    
+    @returns: the mtime of the file (same as returned by time.localtime(...))   
+    @rtype: tuple of 9 ints     
+    """     
+    data = request.getData()    
+    filestat_cache = data.setdefault("filestat_cache", {})      
+    
+    if filestat_cache.has_key(filename):    
+        return filestat_cache[filename]     
+    
+    argdict = {"request": request, "filename": filename,    
+               "mtime": os.stat(filename)}      
+    argdict = run_callback("filestat",      
+                           argdict,     
+                           mappingfunc=lambda x,y:y,    
+                           defaultfunc=lambda x:x)      
+    timetuple = time.localtime(argdict["mtime"][8])     
+    filestat_cache[filename] = timetuple    
      
-     @param request: the Pyblosxom Request object    
-     @type  request: Request     
-     
-     @param filename: the name of the file to stat   
-     @type  filename: string     
-     
-     @returns: the mtime of the file (same as returned by time.localtime(...))   
-     @rtype: tuple of 9 ints     
-     """     
-     data = request.getData()    
-     filestat_cache = data.setdefault("filestat_cache", {})      
-     
-     if filestat_cache.has_key(filename):    
-         return filestat_cache[filename]     
-     
-     argdict = {"request": request, "filename": filename,    
-                "mtime": os.stat(filename)}      
-     argdict = run_callback("filestat",      
-                            argdict,     
-                            mappingfunc=lambda x,y:y,    
-                            defaultfunc=lambda x:x)      
-     timetuple = time.localtime(argdict["mtime"][8])     
-     filestat_cache[filename] = timetuple    
-     
-     return timetuple
+    return timetuple
+
 
 def what_ext(extensions, filepath):
     """
@@ -305,6 +306,28 @@ def what_ext(extensions, filepath):
             return ext
     return None
 
+
+def is_year(s):
+    """
+    Checks s to see if it's likely to be a year or not.  In order to be
+    considered to be a year, it must pass the following criteria:
+
+     1. four digits
+     2. first two digits are either 19 or 20.
+
+    @param s: the string to check for "year-hood"
+    @type  s: string
+
+    @return: 1 if s is likely to be a year or 0 if it is not
+    @rtype: boolean
+    """
+    if not s: return 0
+
+    if len(s) == 4 and s.isdigit() and (s.startswith("19") or s.startswith("20")):
+        return 1
+    return 0
+
+
 def importName(modulename, name):
     """
     Module importer
@@ -465,14 +488,14 @@ def get_cache(request):
 
     return mycache
 
-
+_logger_registry = {}
 def make_logger(filename):
     """
     Create a logging function called log, which logs to the supplied filename
     usage is:
 
-    >>> tools.make_logger('/tmp/pybloxom.log')
-    >>> tools.log('log message')
+    -->>> tools.make_logger('/tmp/pybloxom.log')
+    -->>> tools.log('log message')
 
     @param filename: the name of a file to log to
     @type filename: string
@@ -488,15 +511,26 @@ def make_logger(filename):
             f.write("\n")
             f.close()
     else:
-        logger = logging.getLogger('trackback')
-        hdlr = logging.FileHandler(filename)
-        formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
-        hdlr.setFormatter(formatter)
-        logger.addHandler(hdlr) 
-        logger.setLevel(logging.INFO)
+        global _logger_registry
+        # if all loggers have the same name,
+        # everything is logged to all files.
+        logger_name = os.path.splitext(os.path.basename(filename))[0]
+        # only add one handler per logger
+        if not logger_name in _logger_registry:
+            _logger = logging.getLogger(logger_name)
+            hdlr = logging.FileHandler(filename)
+            formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
+            hdlr.setFormatter(formatter)
+            _logger.addHandler(hdlr)
+            _logger.setLevel(logging.INFO)
+            _logger_registry[logger_name] = _logger
 
-        def log(str):
-            logger.info(str)
+        logger = _logger_registry[logger_name]
+
+        def log(*args):
+            # adjusted to match the 'manual' log func
+            for i in args:
+                logger.info(repr(i))
 
 
 def update_static_entry(cdict, entry_filename):
@@ -554,12 +588,9 @@ def render_url(cdict, pathinfo, querystring=""):
     if not staticdir:
         raise Exception("You must set static_dir in your config file.")
 
-    from Pyblosxom import pyblosxom
-       
-    oldstdout = sys.stdout
+    from Pyblosxom.pyblosxom import PyBlosxom
 
-    req = pyblosxom.Request()
-    req.addHttp({
+    env = {
         "HTTP_USER_AGENT": "static renderer",
         "REQUEST_METHOD": "GET",
         "HTTP_HOST": "localhost",
@@ -568,32 +599,25 @@ def render_url(cdict, pathinfo, querystring=""):
         "REQUEST_URI": pathinfo + "?" + querystring,
         "PATH_INFO": pathinfo,
         "HTTP_REFERER": "",
-        "REMOTE_ADDR": ""
-         })
-    req.addConfiguration(cdict)
-    req.addData( {"STATIC": 1} )
-
-    buffer = StringIO.StringIO()
-    sys.stdout = buffer
-    p = pyblosxom.PyBlosxom(req)
-    p.run()
-    sys.stdout = oldstdout
+        "REMOTE_ADDR": "",
+        "SCRIPT_NAME": "",
+        "wsgi.errors": sys.stderr,
+        "wsgi.input": None
+    }
+    data = {"STATIC": 1}
+    p = PyBlosxom(cdict, env, data)
+    p.run(static=True)
+    response = p.getResponse()
+    response.seek(0)
 
     fn = os.path.normpath(staticdir + os.sep + pathinfo)
     if not os.path.isdir(os.path.dirname(fn)):
         os.makedirs(os.path.dirname(fn))
 
-    # this is cheesy--we need to remove the HTTP headers
-    # from the file.
-    output = buffer.getvalue().splitlines()
-    while 1:
-        if len(output[0].strip()) == 0:
-            break
-        output.pop(0)
-    output.pop(0)
-
+    # by using the response object the cheesy part of removing 
+    # the HTTP headers from the file is history.
     f = open(fn, "w")
-    f.write("\n".join(output))
+    f.write(response.read())
     f.close()
 
  
@@ -635,7 +659,7 @@ def render_url(cdict, pathinfo, querystring=""):
 # that accompanies the win32 modules.
 # 
 # Author: Jonathan Feinberg <jdf@pobox.com>
-# Version: $Id: tools.py,v 1.34 2004/05/24 18:37:07 willhelm Exp $
+# Version: $Id: tools.py,v 1.41 2005/03/04 18:30:57 kaidon Exp $
 
 if os.name == 'nt':
     import win32con
